package net.sit.dialog.srv;

import java.io.File;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;

import net.sit.dialog.cfg.Cfg;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

/**
 * Startklasse des Dialog Servers.
 * 
 * @author Christoph Sandhaus
 *         <p>
 */
public class Server {

	static Logger logger = Logger.getLogger(Server.class.getName());

	/**
	 * Einzige Instanz des Dialog Servers.
	 */
	private static DialogServer dialogServer;

	/**
	 * IP Adresse, an die der Server gebunden wird.
	 */
	private String serverIp;

	/**
	 * Erstellt eine Instanz vom Typ Server.
	 */
	public Server() {
	}

	/**
	 * Starten des Dialog Servers, sofern dieser noch keine Instanz vom Typ
	 * Server gestartet wurde.
	 * 
	 * @param serverIp
	 *            die IP Adresse, an die der Server gebunden wird
	 * @return true bei Erfolg, sonst false<br>
	 *         <br>
	 *         Der Server kann an nur eine IP Adresse gebunden werden. Sollte
	 *         ein Rechner mehrere IP Adressen haben, so lauscht der Server nur
	 *         auf der angegebenen IP Adresse.
	 */
	public synchronized boolean startServer(String serverIp) {

		this.serverIp = serverIp;

		/**
		 * Laden der log4j Konfiguration. Diese Property mu beim Programmstart
		 * als Java VM Property angegeben werden.
		 * <p>
		 * Beispiel:<br>
		 * java -Dlog.profile=cfg/log.properties -D... -jar ...
		 */
		String logProperties = System.getProperty("log.profile");

		/**
		 * Prfen der Property Datei fr log4j. System.getProperty(...) liefert
		 * null, wenn die Property 'log.profile' nicht existiert
		 */
		if (logProperties == null) {
			System.err.println("\nProperty Datei fr log4j nicht gefunden"
					+ " oder sie wurde nicht angegeben.\n"
					+ "Beispielaufruf:\n"
					+ "  java -Dlog.profile=log.properties -jar ...\n");
			System.exit(1);
		} else {
			/** Im Fehlerfall prfen, ob die Property Datei lesbar ist */
			File f = new File(logProperties);
			if (!f.canRead()) {
				System.err.println("\nPropertydatei '" + f.getAbsolutePath()
						+ "' nicht gefunden.\n");
				System.exit(2);
			}
		}
		PropertyConfigurator.configure(logProperties);

		/** Prfen, ob der Server bereits gestartet wurde. */
		if (dialogServer == null) {

			/** Den Security Manager initialisieren */
			initializeSecurityManager();

			try {
				/** Starten der Registry. */
				logger.debug("Versuche Registry auf Port '"
						+ Cfg.RMI_SERVER_PORT + "' zu binden.");
				LocateRegistry.createRegistry(Cfg.RMI_SERVER_PORT);
				logger.info("Registry erstellt.");
			} catch (RemoteException e1) {
				logger.info("RemoteException: " + e1.getMessage());
				logger.info("RMI Registry ist evtl. schon aktiv.");
			}

			/** Den Server initialisieren und an die Registry binden */
			try {
				dialogServer = new DialogServer();
				String srvURL = "rmi://" + serverIp + ":" + Cfg.RMI_SERVER_PORT
						+ "/" + Cfg.RMI_SERVER_NAME;
				logger.debug("Richte RMI Server URL ein: " + srvURL);
				Naming.rebind(srvURL, dialogServer);
				logger.info("Server erfolgreich gestartet.");
			} catch (RemoteException e1) {
				logger.error("RemoteException: " + e1.getMessage());
				e1.printStackTrace();
				return false;
			} catch (MalformedURLException e1) {
				logger.error("MalformedURLException: " + e1.getMessage());
				return false;
			}

		} else {
			logger.info("Server ist schon aktiv. Kein Neustart ...");
		}

		return true;
	}

	/**
	 * Initialisieren des Security Managers
	 */
	private void initializeSecurityManager() {
		String securityFile = System.getProperty("java.security.policy");

		if (securityFile == null) {
			System.err
					.println("\nEs wurde keine Java Policy Datei angegeben.\n"
							+ "Beispielaufruf:\n"
							+ "  java -Djava.security.policy=java.policy_Server -jar ...\n");
			System.exit(3);
		}
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new RMISecurityManager());
			logger.info("Security Manager registriert");
		}
	}

	public static void main(String args[]) {

		/**
		 * Prfen, ob die IP Adresse fr den Server als Argument geliefert
		 * wurde.
		 */
		if (args.length != 1) {
			System.err
					.println("\nBitte die IP Adresse angeben, an die sich der Server binden soll.\n");
		}

		/** Prfen, ob die Versionsnummer abgefragt wurde. */
		if ("-v".equals(args[0])) {
			System.out.println("\nWritten by: Christoph Sandhaus, 2005");
			System.out.println("Lizenz    : GNU Public License");
			System.out.println("Version   : " + Cfg.APP_VERSION + "\n");
			System.exit(0);
		}

		/** Den Server starten. */
		Server srv = new Server();
		srv.startServer(args[0]);
	}
}
